----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch
--
-- Module Name: scalp_aurora_phy_rx_fifo - arch
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.2
-- Description: scalp_aurora_phy_rx_fifo
--
-- Last update: 2020-11-18
--
---------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.axi4_pkg.all;
use work.aurora_status_pkg.all;

entity scalp_aurora_phy_rx_fifo is

    port (
        -- Clocks and Reset
        RXClkxCI           : in  std_ulogic;
        TXClkxCI           : in  std_ulogic;
        RXRstxRANI         : in  t_rx_fifo_reset;
        -- North
        -- RX Data
        NorthRXM2SxDI      : in  t_axi4m2s;
        NorthRXS2MxDO      : out t_axi4s2m;
        -- TX Data
        NorthTXM2SxDO      : out t_axi4m2s;
        NorthTXS2MxDI      : in  t_axi4s2m;
        -- Fifo Status
        NorthFifoStatusxDO : out t_axi4fifo_status;
        -- East
        -- RX Data
        EastRXM2SxDI       : in  t_axi4m2s;
        EastRXS2MxDO       : out t_axi4s2m;
        -- TX Data
        EastTXM2SxDO       : out t_axi4m2s;
        EastTXS2MxDI       : in  t_axi4s2m;
        -- Fifo Status
        EastFifoStatusxDO  : out t_axi4fifo_status;
        -- South
        -- RX Data
        SouthRXM2SxDI      : in  t_axi4m2s;
        SouthRXS2MxDO      : out t_axi4s2m;
        -- TX Data
        SouthTXM2SxDO      : out t_axi4m2s;
        SouthTXS2MxDI      : in  t_axi4s2m;
        -- Fifo Status
        SouthFifoStatusxDO : out t_axi4fifo_status;
        -- West
        -- RX Data
        WestRXM2SxDI       : in  t_axi4m2s;
        WestRXS2MxDO       : out t_axi4s2m;
        -- TX Data
        WestTXM2SxDO       : out t_axi4m2s;
        WestTXS2MxDI       : in  t_axi4s2m;
        -- Fifo Status
        WestFifoStatusxDO  : out t_axi4fifo_status);

end scalp_aurora_phy_rx_fifo;

architecture arch of scalp_aurora_phy_rx_fifo is

    component axis_data_fifo
        port (
            s_axis_aresetn     : in  std_logic;
            s_axis_aclk        : in  std_logic;
            s_axis_tvalid      : in  std_logic;
            s_axis_tready      : out std_logic;
            s_axis_tdata       : in  std_logic_vector((C_AXI4_DATA_SIZE - 1) downto 0);
            s_axis_tkeep       : in  std_logic_vector((C_AXI4_KEEP_SIZE - 1) downto 0);
            s_axis_tlast       : in  std_logic;
            m_axis_aclk        : in  std_logic;
            m_axis_tvalid      : out std_logic;
            m_axis_tready      : in  std_logic;
            m_axis_tdata       : out std_logic_vector((C_AXI4_DATA_SIZE - 1) downto 0);
            m_axis_tkeep       : out std_logic_vector((C_AXI4_KEEP_SIZE - 1) downto 0);
            m_axis_tlast       : out std_logic;
            axis_wr_data_count : out std_logic_vector((C_AXI4_DATA_SIZE - 1) downto 0);
            axis_rd_data_count : out std_logic_vector((C_AXI4_DATA_SIZE - 1) downto 0);
            prog_empty         : out std_logic;
            prog_full          : out std_logic);
    end component;

    signal RXClkxC           : std_ulogic        := '0';
    signal TXClkxC           : std_ulogic        := '0';
    signal RXRstxRAN         : t_rx_fifo_reset   := C_NO_RX_FIFO_RESET;
    -- North
    signal NorthRXM2SxD      : t_axi4m2s         := C_NO_AXI4_M2S;
    signal NorthRXS2MxD      : t_axi4s2m         := C_NO_AXI4_S2M;
    signal NorthTXM2SxD      : t_axi4m2s         := C_NO_AXI4_M2S;
    signal NorthTXS2MxD      : t_axi4s2m         := C_NO_AXI4_S2M;
    signal NorthFifoStatusxD : t_axi4fifo_status := C_NO_AXI4_FIFO_STATUS;
    -- East
    signal EastRXM2SxD       : t_axi4m2s         := C_NO_AXI4_M2S;
    signal EastRXS2MxD       : t_axi4s2m         := C_NO_AXI4_S2M;
    signal EastTXM2SxD       : t_axi4m2s         := C_NO_AXI4_M2S;
    signal EastTXS2MxD       : t_axi4s2m         := C_NO_AXI4_S2M;
    signal EastFifoStatusxD  : t_axi4fifo_status := C_NO_AXI4_FIFO_STATUS;
    -- South
    signal SouthRXM2SxD      : t_axi4m2s         := C_NO_AXI4_M2S;
    signal SouthRXS2MxD      : t_axi4s2m         := C_NO_AXI4_S2M;
    signal SouthTXM2SxD      : t_axi4m2s         := C_NO_AXI4_M2S;
    signal SouthTXS2MxD      : t_axi4s2m         := C_NO_AXI4_S2M;
    signal SouthFifoStatusxD : t_axi4fifo_status := C_NO_AXI4_FIFO_STATUS;
    -- West
    signal WestRXM2SxD       : t_axi4m2s         := C_NO_AXI4_M2S;
    signal WestRXS2MxD       : t_axi4s2m         := C_NO_AXI4_S2M;
    signal WestTXM2SxD       : t_axi4m2s         := C_NO_AXI4_M2S;
    signal WestTXS2MxD       : t_axi4s2m         := C_NO_AXI4_S2M;
    signal WestFifoStatusxD  : t_axi4fifo_status := C_NO_AXI4_FIFO_STATUS;

begin

    AxisDataFifoxB : block is
    begin  -- block AxisDataFifoxB

        EntityIOxB : block is
        begin  -- block EntityIOxB

            RXClkxAS           : RXClkxC            <= RXClkxCI;
            TXClkxAS           : TXClkxC            <= TXClkxCI;
            RXRstxAS           : RXRstxRAN          <= RXRstxRANI;
            -- North
            NorthRXM2SxAS      : NorthRXM2SxD       <= NorthRXM2SxDI;
            NorthRXS2MxAS      : NorthRXS2MxDO      <= NorthRXS2MxD;
            NorthTXM2SxAS      : NorthTXM2SxDO      <= NorthTXM2SxD;
            NorthTXS2MxAS      : NorthTXS2MxD       <= NorthTXS2MxDI;
            NorthFifoStatusxAS : NorthFifoStatusxDO <= NorthFifoStatusxD;
            -- East
            EastRXM2SxAS       : EastRXM2SxD        <= EastRXM2SxDI;
            EastRXS2MxAS       : EastRXS2MxDO       <= EastRXS2MxD;
            EastTXM2SxAS       : EastTXM2SxDO       <= EastTXM2SxD;
            EastTXS2MxAS       : EastTXS2MxD        <= EastTXS2MxDI;
            EastFifoStatusxAS  : EastFifoStatusxDO  <= EastFifoStatusxD;
            -- South
            SouthRXM2SxAS      : SouthRXM2SxD       <= SouthRXM2SxDI;
            SouthRXS2MxAS      : SouthRXS2MxDO      <= SouthRXS2MxD;
            SouthTXM2SxAS      : SouthTXM2SxDO      <= SouthTXM2SxD;
            SouthTXS2MxAS      : SouthTXS2MxD       <= SouthTXS2MxDI;
            SouthFifoStatusxAS : SouthFifoStatusxDO <= SouthFifoStatusxD;
            -- West
            WestRXM2SxAS       : WestRXM2SxD        <= WestRXM2SxDI;
            WestRXS2MxAS       : WestRXS2MxDO       <= WestRXS2MxD;
            WestTXM2SxAS       : WestTXM2SxDO       <= WestTXM2SxD;
            WestTXS2MxAS       : WestTXS2MxD        <= WestTXS2MxDI;
            WestFifoStatusxAS  : WestFifoStatusxDO  <= WestFifoStatusxD;

        end block EntityIOxB;

        NorthAxisDataFifoxI : axis_data_fifo
            port map (
                -- Slave side
                s_axis_aresetn     => RXRstxRAN.NorthxR,
                s_axis_aclk        => RXClkxC,
                s_axis_tdata       => NorthRXM2SxD.DataxD,
                s_axis_tkeep       => NorthRXM2SxD.KeepxD,
                s_axis_tlast       => NorthRXM2SxD.LastxS,
                s_axis_tvalid      => NorthRXM2SxD.ValidxS,
                s_axis_tready      => NorthRXS2MxD.ReadyxS,
                -- Master side
                m_axis_aclk        => TXClkxC,
                m_axis_tdata       => NorthTXM2SxD.DataxD,
                m_axis_tkeep       => NorthTXM2SxD.KeepxD,
                m_axis_tlast       => NorthTXM2SxD.LastxS,
                m_axis_tvalid      => NorthTXM2SxD.ValidxS,
                m_axis_tready      => NorthTXS2MxD.ReadyxS,
                -- Status
                axis_wr_data_count => NorthFifoStatusxD.WrDataCntxD,
                axis_rd_data_count => NorthFifoStatusxD.RdDataCntxD,
                prog_empty         => NorthFifoStatusxD.ProgEmptyxS,
                prog_full          => NorthFifoStatusxD.ProgFullxS);

        EastAxisDataFifoxI : axis_data_fifo
            port map (
                -- Slave side
                s_axis_aresetn     => RXRstxRAN.EastxR,
                s_axis_aclk        => RXClkxC,
                s_axis_tdata       => EastRXM2SxD.DataxD,
                s_axis_tkeep       => EastRXM2SxD.KeepxD,
                s_axis_tlast       => EastRXM2SxD.LastxS,
                s_axis_tvalid      => EastRXM2SxD.ValidxS,
                s_axis_tready      => EastRXS2MxD.ReadyxS,
                -- Master side
                m_axis_aclk        => TXClkxC,
                m_axis_tdata       => EastTXM2SxD.DataxD,
                m_axis_tkeep       => EastTXM2SxD.KeepxD,
                m_axis_tlast       => EastTXM2SxD.LastxS,
                m_axis_tvalid      => EastTXM2SxD.ValidxS,
                m_axis_tready      => EastTXS2MxD.ReadyxS,
                -- Status
                axis_wr_data_count => EastFifoStatusxD.WrDataCntxD,
                axis_rd_data_count => EastFifoStatusxD.RdDataCntxD,
                prog_empty         => EastFifoStatusxD.ProgEmptyxS,
                prog_full          => EastFifoStatusxD.ProgFullxS);

        SouthAxisDataFifoxI : axis_data_fifo
            port map (
                -- Slave side
                s_axis_aresetn     => RXRstxRAN.SouthxR,
                s_axis_aclk        => RXClkxC,
                s_axis_tdata       => SouthRXM2SxD.DataxD,
                s_axis_tkeep       => SouthRXM2SxD.KeepxD,
                s_axis_tlast       => SouthRXM2SxD.LastxS,
                s_axis_tvalid      => SouthRXM2SxD.ValidxS,
                s_axis_tready      => SouthRXS2MxD.ReadyxS,
                -- Master side
                m_axis_aclk        => TXClkxC,
                m_axis_tdata       => SouthTXM2SxD.DataxD,
                m_axis_tkeep       => SouthTXM2SxD.KeepxD,
                m_axis_tlast       => SouthTXM2SxD.LastxS,
                m_axis_tvalid      => SouthTXM2SxD.ValidxS,
                m_axis_tready      => SouthTXS2MxD.ReadyxS,
                -- Status
                axis_wr_data_count => SouthFifoStatusxD.WrDataCntxD,
                axis_rd_data_count => SouthFifoStatusxD.RdDataCntxD,
                prog_empty         => SouthFifoStatusxD.ProgEmptyxS,
                prog_full          => SouthFifoStatusxD.ProgFullxS);

        WestAxisDataFifoxI : axis_data_fifo
            port map (
                -- Slave side
                s_axis_aresetn     => RXRstxRAN.WestxR,
                s_axis_aclk        => RXClkxC,
                s_axis_tdata       => WestRXM2SxD.DataxD,
                s_axis_tkeep       => WestRXM2SxD.KeepxD,
                s_axis_tlast       => WestRXM2SxD.LastxS,
                s_axis_tvalid      => WestRXM2SxD.ValidxS,
                s_axis_tready      => WestRXS2MxD.ReadyxS,
                -- Master side
                m_axis_aclk        => TXClkxC,
                m_axis_tdata       => WestTXM2SxD.DataxD,
                m_axis_tkeep       => WestTXM2SxD.KeepxD,
                m_axis_tlast       => WestTXM2SxD.LastxS,
                m_axis_tvalid      => WestTXM2SxD.ValidxS,
                m_axis_tready      => WestTXS2MxD.ReadyxS,
                -- Status
                axis_wr_data_count => WestFifoStatusxD.WrDataCntxD,
                axis_rd_data_count => WestFifoStatusxD.RdDataCntxD,
                prog_empty         => WestFifoStatusxD.ProgEmptyxS,
                prog_full          => WestFifoStatusxD.ProgFullxS);

    end block AxisDataFifoxB;

end arch;
