----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: scalp_router.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Scalp Router (NoC).
--
-- Last update: 2020-11-30
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

library work;
use work.axi4_pkg.all;
use work.scalp_misc.all;
use work.scalp_utility.all;

entity scalp_router is

    generic (
        C_SCALP_NUMBER_OF_INTERFACE : integer range 0 to 255 := 7;
        C_SCALP_SCHEDULER_STRATEGY  : string                 := "RR");

    port (
        -- System Clock and System nReset
        -----------------------------------------------------------------------
        SysClkxCI          : in  std_ulogic;
        SysRstxRNAI        : in  std_ulogic;
        -- Local Router Network Address
        -----------------------------------------------------------------------
        LocNetAddrxDI      : in  t_scalp_netaddr;
        -- Axi4 Stream
        -----------------------------------------------------------------------
        -- RX Side Vector
        RXAxism2sVectorxDI : in  t_axi4m2s_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0);
        RXAxiss2mVectorxDO : out t_axi4s2m_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0);
        -- TX Side Vector
        TXAxism2sVectorxDO : out t_axi4m2s_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0);
        TXAxiss2mVectorxDI : in  t_axi4s2m_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0);
        -- QoS Vector
        -----------------------------------------------------------------------
        QoSVectorxDI       : in  t_scalp_qos_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0));

end entity scalp_router;

architecture rtl of scalp_router is

    constant C_SCALP_INTERFACE_VECTOR_SIZE     : integer range 0 to 255 := C_SCALP_NUMBER_OF_INTERFACE;
    constant C_SCALP_NEIGHBORS_VECTOR_SIZE     : integer range 0 to 255 := (C_SCALP_NUMBER_OF_INTERFACE - 1);
    -- Multicast Only
    constant C_SCALP_MULTICAST_ONLY            : integer range 0 to 1   := 1;
    constant C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255 := C_SCALP_MULTICAST_ONLY;

    component scalp_router_interface is
        generic (
            C_SCALP_INTERFACE_VECTOR_SIZE     : integer range 0 to 255;
            C_SCALP_INTERFACE_SOURCE_NUMBER   : integer range 0 to 255;
            C_SCALP_NEIGHBORS_VECTOR_SIZE     : integer range 0 to 255;
            C_SCALP_SCHEDULER_ACK_VECTOR_SIZE : integer range 0 to 255;
            C_SCALP_SCHEDULER_STRATEGY        : string);
        port (
            -- System Clock and System nReset
            SysClkxCI         : in  std_ulogic;
            SysRstxRNAI       : in  std_ulogic;
            -- Local Router Network Address
            LocNetAddrxDI     : in  t_scalp_netaddr;
            -- Axi4 Stream
            -----------------------------------------------------------------------
            -- Interface Input Side
            RXAxi4m2sIfxDI    : in  t_axi4m2s;
            RXAxi4s2mIfxDO    : out t_axi4s2m;
            -- Interface Output Side
            TXAxi4m2sIfxDO    : out t_axi4m2s;
            TXAxi4s2mIfxDI    : in  t_axi4s2m;
            -----------------------------------------------------------------------
            -- Neighborhood Links Input Side
            RXAxi4m2sLinksxDI : in  t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            RXAxi4s2mLinksxDO : out t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            -- Neighborhood Links Output Side
            TXAxi4m2sLinksxDO : out t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            TXAxi4s2mLinksxDI : in  t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            -----------------------------------------------------------------------
            -- Booking Vector
            -- Input Side
            BookingVectorxDI  : in  std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            -- Output Side
            BookingVectorxDO  : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            -- Scheduler Ack Vector
            -- Input Side
            SchedulerAckxDI   : in  std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            -- Output Side
            SchedulerAckxDO   : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
            -- QoS Vector
            QoSVectorxDI      : in  t_scalp_qos_vector((C_SCALP_INTERFACE_VECTOR_SIZE - 1) downto 0));
    end component scalp_router_interface;

    -- Signals
    -- Scalp Router
    -- Local Router Network Address
    signal LocNetAddrxD             : t_scalp_netaddr                                                                                                               := C_3D_MIN_SCALP_NETADDR;
    -- Axi4 Stream Interfaces
    signal RXAxism2sVectorxD        : t_axi4m2s_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)                                                                  := (others => C_NO_AXI4_M2S);
    signal RXAxiss2mVectorxD        : t_axi4s2m_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)                                                                  := (others => C_NO_AXI4_S2M);
    signal TXAxism2sVectorxD        : t_axi4m2s_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)                                                                  := (others => C_NO_AXI4_M2S);
    signal TXAxiss2mVectorxD        : t_axi4s2m_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)                                                                  := (others => C_NO_AXI4_S2M);
    -- Scalp QoS Vectors
    signal QoSVectorxD              : t_scalp_qos_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)                                                                := (others => C_SCALP_NO_QOS);
    -- Axi4 Stream Cross-Links with Neighborhood
    signal RXAxi4m2sLinksxD         : t_axi4m2s_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)             := (others => (others => C_NO_AXI4_M2S));
    signal RXAxi4s2mLinksxD         : t_axi4s2m_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)             := (others => (others => C_NO_AXI4_S2M));
    signal TXAxi4m2sLinksxD         : t_axi4m2s_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)             := (others => (others => C_NO_AXI4_M2S));
    signal TXAxi4s2mLinksxD         : t_axi4s2m_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)             := (others => (others => C_NO_AXI4_S2M));
    ---------------------------------------------------------------------------
    -- For Simulation With Vivado Only
    -- Axi4 Stream Cross-Links with Neighborhood
    -- signal RXAxi4m2sLinksxD         : t_axi4m2s_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)                                                           := (others => C_NO_SIM_AXISM2S_VECTOR);
    -- signal RXAxi4s2mLinksxD         : t_axi4s2m_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)                                                           := (others => C_NO_SIM_AXISS2M_VECTOR);
    -- signal TXAxi4m2sLinksxD         : t_axi4m2s_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)                                                           := (others => C_NO_SIM_AXISM2S_VECTOR);
    -- signal TXAxi4s2mLinksxD         : t_axi4s2m_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)                                                           := (others => C_NO_SIM_AXISS2M_VECTOR);
    ---------------------------------------------------------------------------
    -- Scalp Booking Vectors
    signal BookingVectorsInxD       : t_scalp_booking_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)       := (others => (others => '0'));
    signal BookingVectorsOutxD      : t_scalp_booking_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)       := (others => (others => '0'));
    -- Scalp Scheduler Ack Vectors
    signal SchedulerAckVectorsInxD  : t_scalp_scheduler_ack_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0) := (others => (others => '0'));
    signal SchedulerAckVectorsOutxD : t_scalp_scheduler_ack_vector_vector((C_SCALP_NUMBER_OF_INTERFACE - 1) downto 0)((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0) := (others => (others => '0'));

begin  -- architecture rtl

    EntityIOxB : block is
    begin  -- block EntityIOxB
        LocNetAddrxAS      : LocNetAddrxD       <= LocNetAddrxDI;
        RXAxism2sVectorxAS : RXAxism2sVectorxD  <= RXAxism2sVectorxDI;
        RXAxiss2mVectorxAS : RXAxiss2mVectorxDO <= RXAxiss2mVectorxD;
        TXAxism2sVectorxAS : TXAxism2sVectorxDO <= TXAxism2sVectorxD;
        TXAxiss2mVectorxAS : TXAxiss2mVectorxD  <= TXAxiss2mVectorxDI;
        QoSVectorxAS       : QoSVectorxD        <= QoSVectorxDI;
    end block EntityIOxB;

    ScalpRouterInterfacexG : for i in 0 to (C_SCALP_NUMBER_OF_INTERFACE - 1) generate
        assert false report "Generate ScalpRouterInterface #" & integer'image(i) severity note;
        CrossLinksxG : for j in (i + 1) to ((i + 1) + (C_SCALP_NEIGHBORS_VECTOR_SIZE - 1)) generate
            assert false report "Generate Axi4 Stream Cross-Link #" & integer'image(j mod C_SCALP_NUMBER_OF_INTERFACE) severity note;
            assert false report integer'image(i) & "#" &
                integer'image(scalp_matrix_idx(i, j mod C_SCALP_NUMBER_OF_INTERFACE, C_SCALP_NUMBER_OF_INTERFACE)) &
                " => " & integer'image(j mod C_SCALP_NUMBER_OF_INTERFACE) & "#" &
                integer'image(scalp_matrix_idx(j mod C_SCALP_NUMBER_OF_INTERFACE, i, C_SCALP_NUMBER_OF_INTERFACE))
                severity note;

            -- Axi4 Stream Cross-Links with Neighborhood
            RXTXm2sLinksxAS : RXAxi4m2sLinksxD(j mod C_SCALP_NUMBER_OF_INTERFACE)(scalp_matrix_idx(j mod C_SCALP_NUMBER_OF_INTERFACE, i, C_SCALP_NUMBER_OF_INTERFACE))
                <= TXAxi4m2sLinksxD(i)(scalp_matrix_idx(i, j mod C_SCALP_NUMBER_OF_INTERFACE, C_SCALP_NUMBER_OF_INTERFACE));
            RXTXs2mLinksxAS : TXAxi4s2mLinksxD(i)(scalp_matrix_idx(i, j mod C_SCALP_NUMBER_OF_INTERFACE, C_SCALP_NUMBER_OF_INTERFACE))
                <= RXAxi4s2mLinksxD(j mod C_SCALP_NUMBER_OF_INTERFACE)(scalp_matrix_idx(j mod C_SCALP_NUMBER_OF_INTERFACE, i, C_SCALP_NUMBER_OF_INTERFACE));
            -- Booking Vectors Cross-Links with Neighborhood
            BookingVectorsxAS : BookingVectorsInxD(j mod C_SCALP_NUMBER_OF_INTERFACE)(scalp_matrix_idx(j mod C_SCALP_NUMBER_OF_INTERFACE, i, C_SCALP_NUMBER_OF_INTERFACE))
                <= BookingVectorsOutxD(i)(scalp_matrix_idx(i, j mod C_SCALP_NUMBER_OF_INTERFACE, C_SCALP_NUMBER_OF_INTERFACE));
            -- Scalp Scheduler Ack Vectors
            SchedulerAckVectorsxAS : SchedulerAckVectorsInxD(i)(scalp_matrix_idx(i, j mod C_SCALP_NUMBER_OF_INTERFACE, C_SCALP_NUMBER_OF_INTERFACE)) <=
                SchedulerAckVectorsOutxD(j mod C_SCALP_NUMBER_OF_INTERFACE)(scalp_matrix_idx(j mod C_SCALP_NUMBER_OF_INTERFACE, i, C_SCALP_NUMBER_OF_INTERFACE));
        end generate CrossLinksxG;

        ScalpRouterInterfacexI : entity work.scalp_router_interface
            generic map (
                C_SCALP_INTERFACE_VECTOR_SIZE     => C_SCALP_INTERFACE_VECTOR_SIZE,
                C_SCALP_INTERFACE_SOURCE_NUMBER   => i,
                C_SCALP_NEIGHBORS_VECTOR_SIZE     => C_SCALP_NEIGHBORS_VECTOR_SIZE,
                C_SCALP_SCHEDULER_ACK_VECTOR_SIZE => C_SCALP_SCHEDULER_ACK_VECTOR_SIZE,
                C_SCALP_SCHEDULER_STRATEGY        => C_SCALP_SCHEDULER_STRATEGY)
            port map (
                -- System Clock and System nReset
                SysClkxCI         => SysClkxCI,
                SysRstxRNAI       => SysRstxRNAI,
                -- Local Router Network Address
                LocNetAddrxDI     => LocNetAddrxD,
                -- Axi4 Stream
                -----------------------------------------------------------------------
                -- Interface Input Side
                RXAxi4m2sIfxDI    => RXAxism2sVectorxD(i),
                RXAxi4s2mIfxDO    => RXAxiss2mVectorxD(i),
                -- Interface Output Side
                TXAxi4m2sIfxDO    => TXAxism2sVectorxD(i),
                TXAxi4s2mIfxDI    => TXAxiss2mVectorxD(i),
                -----------------------------------------------------------------------
                -- Neighborhood Links Input Side
                RXAxi4m2sLinksxDI => RXAxi4m2sLinksxD(i),
                RXAxi4s2mLinksxDO => RXAxi4s2mLinksxD(i),
                -- Neighborhood Links Output Side
                TXAxi4m2sLinksxDO => TXAxi4m2sLinksxD(i),
                TXAxi4s2mLinksxDI => TXAxi4s2mLinksxD(i),
                -----------------------------------------------------------------------
                -- Booking Vector
                -- Input Side
                BookingVectorxDI  => BookingVectorsInxD(i),
                -- Output Side
                BookingVectorxDO  => BookingVectorsOutxD(i),
                -- Scheduler Ack Vector
                -- Input Side
                SchedulerAckxDI   => SchedulerAckVectorsInxD(i),
                -- Output Side
                SchedulerAckxDO   => SchedulerAckVectorsOutxD(i),
                -- QoS Vector
                QoSVectorxDI      => QoSVectorxD);
    end generate ScalpRouterInterfacexG;

end architecture rtl;
