----------------------------------------------------------------------------------
--                                 _             _
--                                | |_  ___ _ __(_)__ _
--                                | ' \/ -_) '_ \ / _` |
--                                |_||_\___| .__/_\__,_|
--                                         |_|
--
----------------------------------------------------------------------------------
--
-- Company: hepia
-- Author: Joachim Schmidt <joachim.schmidt@hesge.ch>
--
-- Module Name: scalp_rx_side.vhd
-- Target Device: SCALP xc7z015clg485-2
-- Tool version: 2019.1
-- Description: Scalp RX side state machine.
--
-- Last update: 2020-11-30
--
---------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

--use ieee.std_logic_unsigned.all;
--use ieee.std_logic_arith.all;
--use ieee.std_logic_misc.all;

--library UNISIM;
--use UNISIM.VCOMPONENTS.all;

use work.axi4_pkg.all;
use work.scalp_misc.all;

entity scalp_rx_side is

    generic (
        C_SCALP_NEIGHBORS_VECTOR_SIZE : integer range 0 to 255 := 6;
        C_SCALP_SCHEDULER_STRATEGY    : string                 := "RR";
        C_SCALP_SCHEDULER_OUTPUT_REG  : boolean                := true);

    port (
        -- System Clock and Reset
        SysClkxCI            : in  std_ulogic;
        SysRstxRNAI          : in  std_ulogic;
        -- Scheduler Ack Vector
        SchedulerAckxDO      : out std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        -- Booking Vector
        BookingVectorxDI     : in  std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        -- Axi4 Stream
        -- Output Side
        TXAxi4M2SLinkxDO     : out t_axi4m2s;
        TXAxi4S2MLinkxDI     : in  t_axi4s2m;
        -- Input Side
        RXAxi4M2SLinkxDI     : in  t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        RXAxi4S2MLinkxDO     : out t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0);
        -- Debug Interface
        BookingIndexxDO      : out integer range 0 to (C_SCALP_NEIGHBORS_VECTOR_SIZE - 1);
        BookingIndexValidxSO : out std_ulogic);

end entity scalp_rx_side;

architecture behavioral of scalp_rx_side is

    component scalp_scheduler is
        generic (
            C_SCALP_STRATEGY   : string;
            C_SCALP_PORTS      : positive;
            C_SCALP_OUTPUT_REG : boolean);
        port (
            SysClkxCI        : in  std_ulogic;
            SysRstxRNAI      : in  std_ulogic;
            ArbitratexSI     : in  std_ulogic;
            RequestVectorxDI : in  std_ulogic_vector((C_SCALP_PORTS - 1) downto 0);
            ArbitratedxSO    : out std_ulogic;
            GrantVectorxDO   : out std_ulogic_vector((C_SCALP_PORTS - 1) downto 0);
            GrantIndexxDO    : out std_ulogic_vector((scalp_log2ceilnz(C_SCALP_PORTS) - 1) downto 0));
    end component scalp_scheduler;

    component scalp_fifo_double_register is
        port (
            -- System Clock and Reset
            SysClkxCI      : in  std_ulogic;
            SysRstxRNAI    : in  std_ulogic;
            -- Axis Ports (Slave Side)
            Axism2sLinkxDI : in  t_axi4m2s;
            Axiss2mLinkxDO : out t_axi4s2m;
            -- Axis Ports (Master Side)
            Axism2sLinkxDO : out t_axi4m2s;
            Axiss2mLinkxDI : in  t_axi4s2m);
    end component scalp_fifo_double_register;

    -- Enity IO Signals
    signal SchedulerAckxD   : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)                   := (others => '0');
    signal BookingVectorxDP : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)                   := (others => '0');
    signal BookingVectorxDN : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)                   := (others => '0');
    signal TXAxi4M2SLinkxD  : t_axi4m2s                                                                         := C_NO_AXI4_M2S;
    signal TXAxi4S2MLinkxD  : t_axi4s2m                                                                         := C_NO_AXI4_S2M;
    signal RXAxi4M2SLinkxD  : t_axi4m2s_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)                    := (others => C_NO_AXI4_M2S);
    signal RXAxi4S2MLinkxD  : t_axi4s2m_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)                    := (others => C_NO_AXI4_S2M);
    -- RR Scheduler Signals
    signal ArbitratexS      : std_ulogic                                                                        := '0';
    signal RequestVectorxDP : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)                   := (others => '0');
    signal RequestVectorxDN : std_ulogic_vector((C_SCALP_NEIGHBORS_VECTOR_SIZE - 1) downto 0)                   := (others => '0');
    signal ArbitratedxS     : std_ulogic                                                                        := '0';
    -- signal GrantVectorxD    : std_ulogic_vector((C_SCALP_PORTS - 1) downto 0)                   := (others => '0');
    signal GrantIndexxD     : std_ulogic_vector((scalp_log2ceilnz(C_SCALP_NEIGHBORS_VECTOR_SIZE) - 1) downto 0) := (others => '0');
    -- Rx Side States    
    signal RXSideStatexDP   : t_scalp_rx_side_states                                                            := E_SCALP_RX_IDLE;
    signal RXSideStatexDN   : t_scalp_rx_side_states                                                            := E_SCALP_RX_IDLE;
    -- Others    
    signal ReadyInxD        : t_axi4s2m                                                                         := C_NO_AXI4_S2M;
    signal Axism2sInxD      : t_axi4m2s                                                                         := C_NO_AXI4_M2S;
    signal ReadyOutxD       : t_axi4s2m                                                                         := C_NO_AXI4_S2M;
    signal EnPopNPushxS     : std_ulogic                                                                        := '0';

begin  -- architecture behavioral

    -- Asynchronous Statements

    EntityIOxB : block is
    begin  -- block EntityIOxB
        SchedulerAckxAS      : SchedulerAckxDO      <= SchedulerAckxD;
        TXAxi4M2SLinkxAS     : TXAxi4M2SLinkxDO     <= TXAxi4M2SLinkxD;
        TXAxi4S2MLinkxAS     : TXAxi4S2MLinkxD      <= TXAxi4S2MLinkxDI;
        RXAxi4M2SLinkxAS     : RXAxi4M2SLinkxD      <= RXAxi4M2SLinkxDI;
        RXAxi4S2MLinkxAS     : RXAxi4S2MLinkxDO     <= RXAxi4S2MLinkxD;
        -- Debug Interface
        BookingIndexxAS      : BookingIndexxDO      <= to_integer(unsigned(GrantIndexxD));
        BookingIndexValidxAS : BookingIndexValidxSO <= ArbitratedxS;
    end block EntityIOxB;

    AsyncStatementsxB : block is
    begin  -- block AsyncStatementsxB
        ReadyInxAS : ReadyInxD <= TXAxi4S2MLinkxD when EnPopNPushxS = '1' else C_NO_AXI4_S2M;
    end block AsyncStatementsxB;

    -- Synchronous Statements

    ScalpSchedulerxI : entity work.scalp_scheduler
        generic map (
            C_SCALP_STRATEGY   => C_SCALP_SCHEDULER_STRATEGY,
            C_SCALP_PORTS      => C_SCALP_NEIGHBORS_VECTOR_SIZE,
            C_SCALP_OUTPUT_REG => C_SCALP_SCHEDULER_OUTPUT_REG)
        port map (
            SysClkxCI        => SysClkxCI,
            SysRstxRNAI      => SysRstxRNAI,
            ArbitratexSI     => ArbitratexS,
            RequestVectorxDI => RequestVectorxDP,
            ArbitratedxSO    => ArbitratedxS,
            GrantVectorxDO   => open,
            GrantIndexxDO    => GrantIndexxD);

    ScalpFifoDoubleRegisterxI : entity work.scalp_fifo_double_register
        port map (
            -- System Clock and Reset
            SysClkxCI      => SysClkxCI,
            SysRstxRNAI    => SysRstxRNAI,
            -- Axis Ports (Slave Side)
            Axism2sLinkxDI => Axism2sInxD,
            Axiss2mLinkxDO => ReadyOutxD,
            -- Axis Ports (Master Side)
            Axism2sLinkxDO => TXAxi4M2SLinkxD,
            Axiss2mLinkxDI => ReadyInxD);

    UpdateRegxP : process (SysClkxCI, SysRstxRNAI) is
    begin  -- process UpdateRegxP
        if SysRstxRNAI = '0' then
            RXSideStatexDP   <= E_SCALP_RX_IDLE;
            RequestVectorxDP <= (others => '0');
        elsif rising_edge(SysClkxCI) then
            RXSideStatexDP   <= RXSideStatexDN;
            RequestVectorxDP <= RequestVectorxDN;
        end if;
    end process UpdateRegxP;

    RxSideStateMachinexP : process (ArbitratedxS, BookingVectorxDI,
                                    GrantIndexxD, RXAxi4M2SLinkxD,
                                    RXSideStatexDP, ReadyOutxD,
                                    TXAxi4M2SLinkxD.LastxS) is
    begin  -- process RxSideStateMachinexP
        -- Default values
        RXSideStatexDN   <= RXSideStatexDP;
        RequestVectorxDN <= (others => '0');
        SchedulerAckxD   <= (others => '0');
        ArbitratexS      <= '0';
        EnPopNPushxS     <= '0';
        -- All RX Ready to 0
        RXAxi4S2MLinkxD  <= (others => C_NO_AXI4_S2M);
        Axism2sInxD      <= C_NO_AXI4_M2S;

        case RXSideStatexDP is
            when E_SCALP_RX_IDLE =>
                    if to_integer(unsigned(BookingVectorxDI)) /= 0 then
                        RequestVectorxDN <= BookingVectorxDI;
                        RXSideStatexDN   <= E_SCALP_RX_ARBITRATE;
                    end if;

            when E_SCALP_RX_ARBITRATE =>
                    ArbitratexS    <= '1';
                    RXSideStatexDN <= E_SCALP_RX_WAIT_ARBITRATED;

            when E_SCALP_RX_WAIT_ARBITRATED =>
                    if ArbitratedxS = '1' then
                        RXSideStatexDN <= E_SCALP_RX_POP_N_PUSH;
                    end if;

            when E_SCALP_RX_POP_N_PUSH =>
                    RXAxi4S2MLinkxD(to_integer(unsigned(GrantIndexxD))) <= ReadyOutxD;
                    Axism2sInxD                                         <= RXAxi4M2SLinkxD(to_integer(unsigned(GrantIndexxD)));
                    EnPopNPushxS                                        <= '1';

                    if TXAxi4M2SLinkxD.LastxS = '1' then
                        RXSideStatexDN <= E_SCALP_RX_ACK;
                    end if;

            when E_SCALP_RX_ACK =>
                    SchedulerAckxD(to_integer(unsigned(GrantIndexxD))) <= '1';
                    -- if to_integer(unsigned(BookingVectorxDI)) /= 0 then
                    --     RXSideStatexDN <= E_SCALP_RX_ARBITRATE;
                    -- else
                    RXSideStatexDN                                     <= E_SCALP_RX_IDLE;
                    -- end if;

            when others => null;
        end case;
    end process RxSideStateMachinexP;

end architecture behavioral;
